@@ -475,7 +475,7 @@ unsafe fn _llmp_next_msg_ptr(last_msg: *const LlmpMsg) -> *mut LlmpMsg {
 /// May be used to restore the map by id.
 #[derive(Copy, Clone, Debug, Serialize, Deserialize)]
 pub struct LlmpDescription {
-    /// Info about the ShredMap in use
+    /// Info about the [`ShMem`] in use
     shmem: ShMemDescription,
     /// The last message sent or received, depnding on page type
     last_message_offset: Option<u64>,
@@ -1012,7 +1012,7 @@ where
             let shm = self.out_maps.last().unwrap();
             println!(
                 "LLMP_DEBUG: End of page reached for map {} with len {}, sending EOP, bt: {:?}",
-                shm.shmem.id().to_string(),
+                shm.shmem.id(),
                 shm.shmem.len(),
                 bt
             );
@@ -1369,7 +1369,7 @@ where
                     #[cfg(all(feature = "llmp_debug", feature = "std"))]
                     println!(
                         "LLMP_DEBUG: Got a new recv map {} with len {:?}",
-                        self.current_recv_map.shmem.id().to_string(),
+                        self.current_recv_map.shmem.id(),
                         self.current_recv_map.shmem.len()
                     );
                     // After we mapped the new page, return the next message, if available
@@ -1500,7 +1500,7 @@ where
         #[cfg(all(feature = "llmp_debug", feature = "std"))]
         println!(
             "LLMP_DEBUG: Initializing map on {} with size {}",
-            new_map.id().to_string(),
+            new_map.id(),
             new_map.len()
         );
 
@@ -1520,7 +1520,7 @@ where
         //let bt = "<n/a (release)>";
         dbg!(
             "LLMP_DEBUG: Using existing map {} with size {}",
-            existing_map.id().to_string(),
+            existing_map.id(),
             existing_map.len(),
             //bt
         );
