@@ -4,7 +4,7 @@ pub use tuple_list::{tuple_list, tuple_list_type, TupleList};
 
 use core::any::TypeId;
 
-use xxhash_rust::const_xxh3::xxh3_64;
+use xxhash_rust::xxh3::xxh3_64;
 
 #[cfg(feature = "RUSTC_IS_NIGHTLY")]
 /// From https://stackoverflow.com/a/60138532/7658998
@@ -308,10 +308,10 @@ where
 }
 
 /// Allows prepending of values to a tuple
-pub trait Prepend<T>: TupleList {
+pub trait Prepend<T> {
     /// The Resulting [`TupleList`], of an [`Prepend::prepend()`] call,
     /// including the prepended entry.
-    type PreprendResult: TupleList;
+    type PreprendResult;
 
     /// Prepend a value to this tuple, returning a new tuple with prepended value.
     #[must_use]
@@ -319,10 +319,7 @@ pub trait Prepend<T>: TupleList {
 }
 
 /// Implement prepend for tuple list.
-impl<Tail, T> Prepend<T> for Tail
-where
-    Tail: TupleList,
-{
+impl<Tail, T> Prepend<T> for Tail {
     type PreprendResult = Self;
 
     fn prepend(self, value: T) -> (T, Self::PreprendResult) {
@@ -330,11 +327,11 @@ where
     }
 }
 
-/// Append to a `TupeList`
-pub trait Append<T>: TupleList {
+/// Append to a tuple
+pub trait Append<T> {
     /// The Resulting [`TupleList`], of an [`Append::append()`] call,
     /// including the appended entry.
-    type AppendResult: TupleList;
+    type AppendResult;
 
     /// Append Value and return the tuple
     #[must_use]
@@ -353,9 +350,7 @@ impl<T> Append<T> for () {
 /// Implement append for non-empty tuple list.
 impl<Head, Tail, T> Append<T> for (Head, Tail)
 where
-    Self: TupleList,
     Tail: Append<T>,
-    (Head, Tail::AppendResult): TupleList,
 {
     type AppendResult = (Head, Tail::AppendResult);
 
@@ -365,6 +360,38 @@ where
     }
 }
 
+/// Merge two `TupeList`
+pub trait Merge<T> {
+    /// The Resulting [`TupleList`], of an [`Merge::merge()`] call
+    type MergeResult;
+
+    /// Merge and return the merged tuple
+    #[must_use]
+    fn merge(self, value: T) -> Self::MergeResult;
+}
+
+/// Implement merge for an empty tuple list.
+impl<T> Merge<T> for () {
+    type MergeResult = T;
+
+    fn merge(self, value: T) -> Self::MergeResult {
+        value
+    }
+}
+
+/// Implement merge for non-empty tuple list.
+impl<Head, Tail, T> Merge<T> for (Head, Tail)
+where
+    Tail: Merge<T>,
+{
+    type MergeResult = (Head, Tail::MergeResult);
+
+    fn merge(self, value: T) -> Self::MergeResult {
+        let (head, tail) = self;
+        (head, tail.merge(value))
+    }
+}
+
 /// Iterate over a tuple, executing the given `expr` for each element.
 #[macro_export]
 macro_rules! tuple_for_each {
