@@ -80,10 +80,10 @@ mod tests {
     use std::{fs, path::PathBuf};
 
     use crate::{
+        bolts::rands::StdRand,
         corpus::{Corpus, CorpusScheduler, OnDiskCorpus, QueueCorpusScheduler, Testcase},
         inputs::bytes::BytesInput,
-        state::{HasCorpus, State},
-        utils::StdRand,
+        state::{HasCorpus, StdState},
     };
 
     #[test]
@@ -103,7 +103,7 @@ mod tests {
             OnDiskCorpus::<BytesInput>::new(PathBuf::from("target/.test/fancy/objective/path"))
                 .unwrap();
 
-        let mut state = State::new(rand, q, (), objective_q, ());
+        let mut state = StdState::new(rand, q, objective_q, ());
 
         let next_idx = scheduler.next(&mut state).unwrap();
         let filename = state
@@ -114,7 +114,7 @@ mod tests {
             .filename()
             .as_ref()
             .unwrap()
-            .to_owned();
+            .clone();
 
         assert_eq!(filename, "target/.test/fancy/path/fancyfile");
 
