@@ -14,12 +14,10 @@ use crate::{
     executors::ExitKind,
     inputs::Input,
     observers::{ObserversTuple, TimeObserver},
+    state::HasClientPerfStats,
     Error,
 };
 
-#[cfg(feature = "introspection")]
-use crate::stats::NUM_FEEDBACKS;
-
 use core::{marker::PhantomData, time::Duration};
 
 /// Feedbacks evaluate the observers.
@@ -28,6 +26,7 @@ use core::{marker::PhantomData, time::Duration};
 pub trait Feedback<I, S>: Named
 where
     I: Input,
+    S: HasClientPerfStats,
 {
     /// `is_interesting ` return if an input is worth the addition to the corpus
     fn is_interesting<EM, OT>(
@@ -44,15 +43,13 @@ where
 
     #[cfg(feature = "introspection")]
     #[allow(clippy::too_many_arguments)]
-    fn is_interesting_with_perf<EM, OT>(
+    fn is_interesting_introspection<EM, OT>(
         &mut self,
         state: &mut S,
         manager: &mut EM,
         input: &I,
         observers: &OT,
         exit_kind: &ExitKind,
-        feedback_stats: &mut [u64; NUM_FEEDBACKS],
-        feedback_index: usize,
     ) -> Result<bool, Error>
     where
         EM: EventFirer<I, S>,
@@ -67,10 +64,10 @@ where
         // Get the elapsed time for checking this feedback
         let elapsed = crate::bolts::cpu::read_time_counter() - start_time;
 
-        // TODO: A more meaningful way to get perf for each feedback
-
         // Add this stat to the feedback metrics
-        feedback_stats[feedback_index] = elapsed;
+        state
+            .introspection_stats_mut()
+            .update_feedback(self.name(), elapsed);
 
         ret
     }
@@ -114,6 +111,7 @@ where
     B: Feedback<I, S>,
     FL: FeedbackLogic<A, B, I, S>,
     I: Input,
+    S: HasClientPerfStats,
 {
     pub first: A,
     pub second: B,
@@ -127,6 +125,7 @@ where
     B: Feedback<I, S>,
     FL: FeedbackLogic<A, B, I, S>,
     I: Input,
+    S: HasClientPerfStats,
 {
     fn name(&self) -> &str {
         self.name.as_ref()
@@ -139,6 +138,7 @@ where
     B: Feedback<I, S>,
     FL: FeedbackLogic<A, B, I, S>,
     I: Input,
+    S: HasClientPerfStats,
 {
     pub fn new(first: A, second: B) -> Self {
         let name = format!("{} ({},{})", FL::name(), first.name(), second.name());
@@ -157,6 +157,7 @@ where
     B: Feedback<I, S>,
     FL: FeedbackLogic<A, B, I, S>,
     I: Input,
+    S: HasClientPerfStats,
 {
     fn is_interesting<EM, OT>(
         &mut self,
@@ -182,21 +183,19 @@ where
     }
 
     #[cfg(feature = "introspection")]
-    fn is_interesting_with_perf<EM, OT>(
+    fn is_interesting_introspection<EM, OT>(
         &mut self,
         state: &mut S,
         manager: &mut EM,
         input: &I,
         observers: &OT,
         exit_kind: &ExitKind,
-        feedback_stats: &mut [u64; NUM_FEEDBACKS],
-        feedback_index: usize,
     ) -> Result<bool, Error>
     where
         EM: EventFirer<I, S>,
         OT: ObserversTuple<I, S>,
     {
-        FL::is_pair_interesting_with_perf(
+        FL::is_pair_interesting_introspection(
             &mut self.first,
             &mut self.second,
             state,
@@ -204,8 +203,6 @@ where
             input,
             observers,
             exit_kind,
-            feedback_stats,
-            feedback_index,
         )
     }
 
@@ -227,6 +224,7 @@ where
     A: Feedback<I, S>,
     B: Feedback<I, S>,
     I: Input,
+    S: HasClientPerfStats,
 {
     fn name() -> &'static str;
 
@@ -245,7 +243,7 @@ where
 
     #[cfg(feature = "introspection")]
     #[allow(clippy::too_many_arguments)]
-    fn is_pair_interesting_with_perf<EM, OT>(
+    fn is_pair_interesting_introspection<EM, OT>(
         first: &mut A,
         second: &mut B,
         state: &mut S,
@@ -253,8 +251,6 @@ where
         input: &I,
         observers: &OT,
         exit_kind: &ExitKind,
-        feedback_stats: &mut [u64; NUM_FEEDBACKS],
-        feedback_index: usize,
     ) -> Result<bool, Error>
     where
         EM: EventFirer<I, S>,
@@ -271,6 +267,7 @@ where
     A: Feedback<I, S>,
     B: Feedback<I, S>,
     I: Input,
+    S: HasClientPerfStats,
 {
     fn name() -> &'static str {
         "Eager OR"
@@ -295,7 +292,7 @@ where
     }
 
     #[cfg(feature = "introspection")]
-    fn is_pair_interesting_with_perf<EM, OT>(
+    fn is_pair_interesting_introspection<EM, OT>(
         first: &mut A,
         second: &mut B,
         state: &mut S,
@@ -303,33 +300,15 @@ where
         input: &I,
         observers: &OT,
         exit_kind: &ExitKind,
-        feedback_stats: &mut [u64; NUM_FEEDBACKS],
-        feedback_index: usize,
     ) -> Result<bool, Error>
     where
         EM: EventFirer<I, S>,
         OT: ObserversTuple<I, S>,
     {
         // Execute this feedback
-        let a = first.is_interesting_with_perf(
-            state,
-            manager,
-            input,
-            observers,
-            exit_kind,
-            feedback_stats,
-            feedback_index,
-        )?;
+        let a = first.is_interesting_introspection(state, manager, input, observers, exit_kind)?;
 
-        let b = second.is_interesting_with_perf(
-            state,
-            manager,
-            input,
-            observers,
-            exit_kind,
-            feedback_stats,
-            feedback_index + 1,
-        )?;
+        let b = second.is_interesting_introspection(state, manager, input, observers, exit_kind)?;
         Ok(a || b)
     }
 }
@@ -339,6 +318,7 @@ where
     A: Feedback<I, S>,
     B: Feedback<I, S>,
     I: Input,
+    S: HasClientPerfStats,
 {
     fn name() -> &'static str {
         "Fast OR"
@@ -366,7 +346,7 @@ where
     }
 
     #[cfg(feature = "introspection")]
-    fn is_pair_interesting_with_perf<EM, OT>(
+    fn is_pair_interesting_introspection<EM, OT>(
         first: &mut A,
         second: &mut B,
         state: &mut S,
@@ -374,37 +354,19 @@ where
         input: &I,
         observers: &OT,
         exit_kind: &ExitKind,
-        feedback_stats: &mut [u64; NUM_FEEDBACKS],
-        feedback_index: usize,
     ) -> Result<bool, Error>
     where
         EM: EventFirer<I, S>,
         OT: ObserversTuple<I, S>,
     {
         // Execute this feedback
-        let a = first.is_interesting_with_perf(
-            state,
-            manager,
-            input,
-            observers,
-            exit_kind,
-            feedback_stats,
-            feedback_index,
-        )?;
+        let a = first.is_interesting_introspection(state, manager, input, observers, exit_kind)?;
 
         if a {
             return Ok(true);
         }
 
-        second.is_interesting_with_perf(
-            state,
-            manager,
-            input,
-            observers,
-            exit_kind,
-            feedback_stats,
-            feedback_index + 1,
-        )
+        second.is_interesting_introspection(state, manager, input, observers, exit_kind)
     }
 }
 
@@ -413,6 +375,7 @@ where
     A: Feedback<I, S>,
     B: Feedback<I, S>,
     I: Input,
+    S: HasClientPerfStats,
 {
     fn name() -> &'static str {
         "Eager AND"
@@ -437,7 +400,7 @@ where
     }
 
     #[cfg(feature = "introspection")]
-    fn is_pair_interesting_with_perf<EM, OT>(
+    fn is_pair_interesting_introspection<EM, OT>(
         first: &mut A,
         second: &mut B,
         state: &mut S,
@@ -445,33 +408,15 @@ where
         input: &I,
         observers: &OT,
         exit_kind: &ExitKind,
-        feedback_stats: &mut [u64; NUM_FEEDBACKS],
-        feedback_index: usize,
     ) -> Result<bool, Error>
     where
         EM: EventFirer<I, S>,
         OT: ObserversTuple<I, S>,
     {
         // Execute this feedback
-        let a = first.is_interesting_with_perf(
-            state,
-            manager,
-            input,
-            observers,
-            exit_kind,
-            feedback_stats,
-            feedback_index,
-        )?;
+        let a = first.is_interesting_introspection(state, manager, input, observers, exit_kind)?;
 
-        let b = second.is_interesting_with_perf(
-            state,
-            manager,
-            input,
-            observers,
-            exit_kind,
-            feedback_stats,
-            feedback_index + 1,
-        )?;
+        let b = second.is_interesting_introspection(state, manager, input, observers, exit_kind)?;
         Ok(a && b)
     }
 }
@@ -481,6 +426,7 @@ where
     A: Feedback<I, S>,
     B: Feedback<I, S>,
     I: Input,
+    S: HasClientPerfStats,
 {
     fn name() -> &'static str {
         "Fast AND"
@@ -508,7 +454,7 @@ where
     }
 
     #[cfg(feature = "introspection")]
-    fn is_pair_interesting_with_perf<EM, OT>(
+    fn is_pair_interesting_introspection<EM, OT>(
         first: &mut A,
         second: &mut B,
         state: &mut S,
@@ -516,37 +462,19 @@ where
         input: &I,
         observers: &OT,
         exit_kind: &ExitKind,
-        feedback_stats: &mut [u64; NUM_FEEDBACKS],
-        feedback_index: usize,
     ) -> Result<bool, Error>
     where
         EM: EventFirer<I, S>,
         OT: ObserversTuple<I, S>,
     {
         // Execute this feedback
-        let a = first.is_interesting_with_perf(
-            state,
-            manager,
-            input,
-            observers,
-            exit_kind,
-            feedback_stats,
-            feedback_index,
-        )?;
+        let a = first.is_interesting_introspection(state, manager, input, observers, exit_kind)?;
 
         if !a {
             return Ok(false);
         }
 
-        second.is_interesting_with_perf(
-            state,
-            manager,
-            input,
-            observers,
-            exit_kind,
-            feedback_stats,
-            feedback_index + 1,
-        )
+        second.is_interesting_introspection(state, manager, input, observers, exit_kind)
     }
 }
 
@@ -573,6 +501,7 @@ pub struct NotFeedback<A, I, S>
 where
     A: Feedback<I, S>,
     I: Input,
+    S: HasClientPerfStats,
 {
     /// The feedback to invert
     pub first: A,
@@ -585,6 +514,7 @@ impl<A, I, S> Feedback<I, S> for NotFeedback<A, I, S>
 where
     A: Feedback<I, S>,
     I: Input,
+    S: HasClientPerfStats,
 {
     fn is_interesting<EM, OT>(
         &mut self,
@@ -618,6 +548,7 @@ impl<A, I, S> Named for NotFeedback<A, I, S>
 where
     A: Feedback<I, S>,
     I: Input,
+    S: HasClientPerfStats,
 {
     #[inline]
     fn name(&self) -> &str {
@@ -629,6 +560,7 @@ impl<A, I, S> NotFeedback<A, I, S>
 where
     A: Feedback<I, S>,
     I: Input,
+    S: HasClientPerfStats,
 {
     /// Creates a new [`NotFeedback`].
     pub fn new(first: A) -> Self {
@@ -696,6 +628,7 @@ macro_rules! feedback_not {
 impl<I, S> Feedback<I, S> for ()
 where
     I: Input,
+    S: HasClientPerfStats,
 {
     fn is_interesting<EM, OT>(
         &mut self,
@@ -727,6 +660,7 @@ pub struct CrashFeedback {}
 impl<I, S> Feedback<I, S> for CrashFeedback
 where
     I: Input,
+    S: HasClientPerfStats,
 {
     fn is_interesting<EM, OT>(
         &mut self,
@@ -776,6 +710,7 @@ pub struct TimeoutFeedback {}
 impl<I, S> Feedback<I, S> for TimeoutFeedback
 where
     I: Input,
+    S: HasClientPerfStats,
 {
     fn is_interesting<EM, OT>(
         &mut self,
@@ -830,6 +765,7 @@ pub struct TimeFeedback {
 impl<I, S> Feedback<I, S> for TimeFeedback
 where
     I: Input,
+    S: HasClientPerfStats,
 {
     fn is_interesting<EM, OT>(
         &mut self,
