@@ -14,18 +14,15 @@ use crate::{
         serdeany::{SerdeAny, SerdeAnyMap},
     },
     corpus::Corpus,
-    events::{Event, EventManager, LogSeverity},
+    events::{Event, EventFirer, LogSeverity},
     feedbacks::FeedbackStatesTuple,
-    fuzzer::Evaluator,
+    fuzzer::{Evaluator, ExecuteInputResult},
     generators::Generator,
     inputs::Input,
     stats::ClientPerfStats,
     Error,
 };
 
-#[cfg(feature = "std")]
-use crate::inputs::bytes::BytesInput;
-
 /// The maximum size of a testcase
 pub const DEFAULT_MAX_SIZE: usize = 1_048_576;
 
@@ -323,7 +320,7 @@ where
     }
 
     fn set_max_size(&mut self, max_size: usize) {
-        self.max_size = max_size
+        self.max_size = max_size;
     }
 }
 
@@ -349,23 +346,27 @@ where
 }
 
 #[cfg(feature = "std")]
-impl<C, FT, R, SC> StdState<C, FT, BytesInput, R, SC>
+impl<C, FT, I, R, SC> StdState<C, FT, I, R, SC>
 where
-    C: Corpus<BytesInput>,
+    C: Corpus<I>,
+    I: Input,
     R: Rand,
     FT: FeedbackStatesTuple,
-    SC: Corpus<BytesInput>,
+    SC: Corpus<I>,
 {
     /// loads inputs from a directory
+    /// If `forced` is `true`, the value will be loaded,
+    /// even if it's not considered to be `interesting`.
     fn load_from_directory<E, EM, Z>(
         &mut self,
         fuzzer: &mut Z,
         executor: &mut E,
         manager: &mut EM,
         in_dir: &Path,
+        forced: bool,
     ) -> Result<(), Error>
     where
-        Z: Evaluator<E, EM, BytesInput, Self>,
+        Z: Evaluator<E, EM, I, Self>,
     {
         for entry in fs::read_dir(in_dir)? {
             let entry = entry?;
@@ -380,20 +381,68 @@ where
 
             if attr.is_file() && attr.len() > 0 {
                 println!("Loading file {:?} ...", &path);
-                let bytes = fs::read(&path)?;
-                let input = BytesInput::new(bytes);
-                let (is_interesting, _) = fuzzer.evaluate_input(self, executor, manager, input)?;
-                if !is_interesting {
-                    println!("File {:?} was not interesting, skipped.", &path);
+                let input = I::from_file(&path)?;
+                if forced {
+                    let _ = fuzzer.add_input(self, executor, manager, input)?;
+                } else {
+                    let (res, _) = fuzzer.evaluate_input(self, executor, manager, input)?;
+                    if res == ExecuteInputResult::None {
+                        println!("File {:?} was not interesting, skipped.", &path);
+                    }
                 }
             } else if attr.is_dir() {
-                self.load_from_directory(fuzzer, executor, manager, &path)?;
+                self.load_from_directory(fuzzer, executor, manager, &path, forced)?;
             }
         }
 
         Ok(())
     }
 
+    /// Loads initial inputs from the passed-in `in_dirs`.
+    /// If `forced` is true, will add all testcases, no matter what.
+    fn load_initial_inputs_internal<E, EM, Z>(
+        &mut self,
+        fuzzer: &mut Z,
+        executor: &mut E,
+        manager: &mut EM,
+        in_dirs: &[PathBuf],
+        forced: bool,
+    ) -> Result<(), Error>
+    where
+        Z: Evaluator<E, EM, I, Self>,
+        EM: EventFirer<I, Self>,
+    {
+        for in_dir in in_dirs {
+            self.load_from_directory(fuzzer, executor, manager, in_dir, forced)?;
+        }
+        manager.fire(
+            self,
+            Event::Log {
+                severity_level: LogSeverity::Debug,
+                message: format!("Loaded {} initial testcases.", self.corpus().count()), // get corpus count
+                phantom: PhantomData,
+            },
+        )?;
+        Ok(())
+    }
+
+    /// Loads all intial inputs, even if they are not consiered `intesting`.
+    /// This is rarely the right method, use `load_initial_inputs`,
+    /// and potentially fix your `Feedback`, instead.
+    pub fn load_initial_inputs_forced<E, EM, Z>(
+        &mut self,
+        fuzzer: &mut Z,
+        executor: &mut E,
+        manager: &mut EM,
+        in_dirs: &[PathBuf],
+    ) -> Result<(), Error>
+    where
+        Z: Evaluator<E, EM, I, Self>,
+        EM: EventFirer<I, Self>,
+    {
+        self.load_initial_inputs_internal(fuzzer, executor, manager, in_dirs, true)
+    }
+
     /// Loads initial inputs from the passed-in `in_dirs`.
     pub fn load_initial_inputs<E, EM, Z>(
         &mut self,
@@ -403,22 +452,10 @@ where
         in_dirs: &[PathBuf],
     ) -> Result<(), Error>
     where
-        Z: Evaluator<E, EM, BytesInput, Self>,
-        EM: EventManager<E, BytesInput, Self, Z>,
+        Z: Evaluator<E, EM, I, Self>,
+        EM: EventFirer<I, Self>,
     {
-        for in_dir in in_dirs {
-            self.load_from_directory(fuzzer, executor, manager, in_dir)?;
-        }
-        manager.fire(
-            self,
-            Event::Log {
-                severity_level: LogSeverity::Debug,
-                message: format!("Loaded {} initial testcases.", self.corpus().count()), // get corpus count
-                phantom: PhantomData,
-            },
-        )?;
-        manager.process(fuzzer, self, executor)?;
-        Ok(())
+        self.load_initial_inputs_internal(fuzzer, executor, manager, in_dirs, false)
     }
 }
 
@@ -442,13 +479,13 @@ where
     where
         G: Generator<I, R>,
         Z: Evaluator<E, EM, I, Self>,
-        EM: EventManager<E, I, Self, Z>,
+        EM: EventFirer<I, Self>,
     {
         let mut added = 0;
         for _ in 0..num {
             let input = generator.generate(self.rand_mut())?;
-            let (is_interesting, _) = fuzzer.evaluate_input(self, executor, manager, input)?;
-            if is_interesting {
+            let (res, _) = fuzzer.evaluate_input(self, executor, manager, input)?;
+            if res != ExecuteInputResult::None {
                 added += 1;
             }
         }
@@ -460,7 +497,6 @@ where
                 phantom: PhantomData,
             },
         )?;
-        manager.process(fuzzer, self, executor)?;
         Ok(())
     }
 
