@@ -19,12 +19,6 @@ use crate::bolts::current_time;
 
 const CLIENT_STATS_TIME_WINDOW_SECS: u64 = 5; // 5 seconds
 
-/// Number of stages in the fuzzer
-pub(crate) const NUM_STAGES: usize = 8;
-
-/// Number of feedback mechanisms to measure for performance
-pub(crate) const NUM_FEEDBACKS: usize = 16;
-
 /// User-defined stats types
 #[derive(Serialize, Deserialize, Debug, Clone)]
 pub enum UserStats {
@@ -294,7 +288,7 @@ where
         {
             // Print the client performance stats.
             let fmt = format!(
-                "Client {:03}: {}",
+                "Client {:03}:\n{}",
                 sender_id, self.client_stats[sender_id as usize].introspection_stats
             );
             (self.print_fn)(fmt);
@@ -358,7 +352,7 @@ macro_rules! mark_feedback_time {
 }
 
 /// Client performance statistics
-#[derive(Serialize, Deserialize, Debug, Copy, Clone)]
+#[derive(Serialize, Deserialize, Debug, Clone)]
 pub struct ClientPerfStats {
     /// Starting counter (in clock cycles from [`cpu::read_time_counter`])
     start_time: u64,
@@ -375,26 +369,22 @@ pub struct ClientPerfStats {
     /// Current stage index to write the next stage benchmark time
     curr_stage: u8,
 
-    /// Current feedback index to write the next feedback benchmark time
-    curr_feedback: u8,
-
     /// Flag to dictate this stage is in use. Used during printing to not print the empty
     /// stages if they are not in use.
-    stages_used: [bool; NUM_STAGES],
+    stages_used: Vec<bool>,
 
-    // TODO(andrea) use an hashmap and indetify feaures using a &'static str
     /// Clock cycles spent in the the various features of each stage
-    stages: [[u64; PerfFeature::Count as usize]; NUM_STAGES],
+    stages: Vec<[u64; PerfFeature::Count as usize]>,
 
     /// Clock cycles spent in each feedback mechanism of the fuzzer.
-    feedbacks: [u64; NUM_FEEDBACKS],
+    feedbacks: HashMap<String, u64>,
 
     /// Current time set by `start_timer`
     timer_start: Option<u64>,
 }
 
 /// Various features that are measured for performance
-#[derive(Serialize, Deserialize, Debug, Copy, Clone)]
+#[derive(Serialize, Deserialize, Debug, Clone)]
 #[repr(u8)]
 pub enum PerfFeature {
     /// Getting an input from the corpus
@@ -477,7 +467,7 @@ impl From<usize> for PerfFeature {
 
 /// Number of features we can measure for performance
 #[cfg(feature = "introspection")]
-const NUM_PERF_FEATURES: usize = PerfFeature::Count as usize;
+pub const NUM_PERF_FEATURES: usize = PerfFeature::Count as usize;
 
 #[cfg(feature = "introspection")]
 impl ClientPerfStats {
@@ -493,10 +483,9 @@ impl ClientPerfStats {
             scheduler: 0,
             manager: 0,
             curr_stage: 0,
-            curr_feedback: 0,
-            stages: [[0; NUM_PERF_FEATURES]; NUM_STAGES],
-            stages_used: [false; NUM_STAGES],
-            feedbacks: [0; NUM_FEEDBACKS],
+            stages: vec![],
+            stages_used: vec![],
+            feedbacks: HashMap::new(),
             timer_start: None,
         }
     }
@@ -514,12 +503,12 @@ impl ClientPerfStats {
     }
 
     /// Update the current [`ClientPerfStats`] with the given [`ClientPerfStats`]
-    pub fn update(&mut self, stats: ClientPerfStats) {
+    pub fn update(&mut self, stats: &ClientPerfStats) {
         self.set_current_time(stats.current_time);
         self.update_scheduler(stats.scheduler);
         self.update_manager(stats.manager);
-        self.update_stages(stats.stages);
-        self.update_feedbacks(stats.feedbacks);
+        self.update_stages(&stats.stages);
+        self.update_feedbacks(&stats.feedbacks);
     }
 
     /// Gets the elapsed time since the internal timer started. Resets the timer when
@@ -578,33 +567,6 @@ impl ClientPerfStats {
         self.update_feature(feature, elapsed);
     }
 
-    /// Update the time spent in the current [`Feedback`] with the elapsed time that we
-    /// have seen
-    pub fn mark_feedback_time(&mut self) {
-        // Sanity check that these stats have enough room for these benchmarks
-        assert!(
-            (self.curr_feedback as usize) < NUM_FEEDBACKS,
-            "Current fuzzer has more 
-                stages than the `ClientPerfStats` supports ({}). Please update the 
-                NUM_FEEDBACKS const value in src/stats/mod.rs and rebuild",
-            NUM_FEEDBACKS
-        );
-
-        // Get the current elapsed time
-        let elapsed = self.mark_time();
-
-        // Get a `usize` for the index
-        let index: usize = self.curr_feedback.try_into().unwrap();
-
-        // Update the current feedback's time with the given time
-        self.feedbacks[index] = self.feedbacks[index]
-            .checked_add(elapsed)
-            .expect("mark_feedback_time overflow");
-
-        // Increment the feedback index to the next feedback
-        self.curr_feedback += 1;
-    }
-
     /// Add the given `time` to the `scheduler` stats
     #[inline]
     pub fn update_scheduler(&mut self, time: u64) {
@@ -637,23 +599,32 @@ impl ClientPerfStats {
         self.curr_stage = 0;
     }
 
-    /// Reset the feedback index counter to zero
-    #[inline]
-    pub fn reset_feedback_index(&mut self) {
-        self.curr_feedback = 0;
+    /// Update the time spent in the feedback
+    pub fn update_feedback(&mut self, name: &str, time: u64) {
+        self.feedbacks.insert(
+            name.into(),
+            self.feedbacks
+                .get(name)
+                .unwrap_or(&0)
+                .checked_add(time)
+                .expect("update_feedback overflow"),
+        );
     }
 
-    /// Update the time spent in the feedbacks
-    pub fn update_feedbacks(&mut self, feedbacks: [u64; NUM_FEEDBACKS]) {
-        for (feedback_index, feedback) in feedbacks.iter().enumerate() {
-            self.feedbacks[feedback_index] = self.feedbacks[feedback_index]
-                .checked_add(*feedback)
-                .expect("update_feedback overflow");
+    /// Update the time spent in all the feedbacks
+    pub fn update_feedbacks(&mut self, feedbacks: &HashMap<String, u64>) {
+        for (key, value) in feedbacks {
+            self.update_feedback(key, *value);
         }
     }
 
     /// Update the time spent in the stages
-    pub fn update_stages(&mut self, stages: [[u64; NUM_PERF_FEATURES]; NUM_STAGES]) {
+    pub fn update_stages(&mut self, stages: &[[u64; PerfFeature::Count as usize]]) {
+        if self.stages.len() < stages.len() {
+            self.stages
+                .resize(stages.len(), [0; PerfFeature::Count as usize]);
+            self.stages_used.resize(stages.len(), false);
+        }
         for (stage_index, features) in stages.iter().enumerate() {
             for (feature_index, feature) in features.iter().enumerate() {
                 self.stages[stage_index][feature_index] = self.stages[stage_index][feature_index]
@@ -665,21 +636,18 @@ impl ClientPerfStats {
 
     /// Update the given [`PerfFeature`] with the given `time`
     pub fn update_feature(&mut self, feature: PerfFeature, time: u64) {
-        // Sanity check that these stats have enough room for these benchmarks
-        assert!(
-            (self.curr_stage as usize) < NUM_STAGES,
-            "Current fuzzer has more stages 
-                than the `ClientPerfStats` supports ({}). Please update the NUM_STAGES
-                const value in src/stats/mod.rs and rebuild",
-            NUM_STAGES
-        );
-
         // Get the current stage index as `usize`
         let stage_index: usize = self.curr_stage.try_into().unwrap();
 
         // Get the index of the given feature
         let feature_index: usize = feature.try_into().unwrap();
 
+        if stage_index >= self.stages.len() {
+            self.stages
+                .resize(stage_index + 1, [0; PerfFeature::Count as usize]);
+            self.stages_used.resize(stage_index + 1, false);
+        }
+
         // Update the given feature
         self.stages[stage_index][feature_index] = self.stages[stage_index][feature_index]
             .checked_add(time)
@@ -688,6 +656,41 @@ impl ClientPerfStats {
         // Set that the current stage is being used
         self.stages_used[stage_index] = true;
     }
+
+    /// The elapsed cycles (or time)
+    #[must_use]
+    pub fn elapsed_cycles(&self) -> u64 {
+        self.current_time - self.start_time
+    }
+
+    /// The amount of cycles the `manager` did
+    #[must_use]
+    pub fn manager_cycles(&self) -> u64 {
+        self.manager
+    }
+
+    /// The amount of cycles the `scheduler` did
+    #[must_use]
+    pub fn scheduler_cycles(&self) -> u64 {
+        self.scheduler
+    }
+
+    /// Iterator over all used stages
+    pub fn used_stages(
+        &self,
+    ) -> impl Iterator<Item = (usize, &[u64; PerfFeature::Count as usize])> {
+        let used = self.stages_used.clone();
+        self.stages
+            .iter()
+            .enumerate()
+            .filter(move |(stage_index, _)| used[*stage_index as usize])
+    }
+
+    /// A map of all `feedbacks`
+    #[must_use]
+    pub fn feedbacks(&self) -> &HashMap<String, u64> {
+        &self.feedbacks
+    }
 }
 
 #[cfg(feature = "introspection")]
@@ -695,7 +698,7 @@ impl core::fmt::Display for ClientPerfStats {
     #[allow(clippy::cast_precision_loss)]
     fn fmt(&self, f: &mut core::fmt::Formatter) -> Result<(), core::fmt::Error> {
         // Calculate the elapsed time from the stats
-        let elapsed: f64 = (self.current_time - self.start_time) as f64;
+        let elapsed: f64 = self.elapsed_cycles() as f64;
 
         // Calculate the percentages for each benchmark
         let scheduler_percent = self.scheduler as f64 / elapsed;
@@ -709,17 +712,13 @@ impl core::fmt::Display for ClientPerfStats {
         // Create the formatted string
         writeln!(
             f,
-            "Scheduler: {:4.2} | Manager: {:4.2} | Stages:",
+            "  {:6.4}: Scheduler\n  {:6.4}: Manager",
             scheduler_percent, manager_percent
         )?;
 
         // Calculate each stage
-        for (stage_index, features) in self.stages.iter().enumerate() {
-            // Make sure this stage is actually used before dumping its information
-            if !self.stages_used[stage_index as usize] {
-                continue;
-            }
-
+        // Make sure we only iterate over used stages
+        for (stage_index, features) in self.used_stages() {
             // Write the stage header
             writeln!(f, "  Stage {}:", stage_index)?;
 
@@ -741,29 +740,27 @@ impl core::fmt::Display for ClientPerfStats {
                 // Write the percentage for this feature
                 writeln!(f, "    {:6.4}: {:?}", feature_percent, feature)?;
             }
-
-            for (feedback_index, feedback) in self.feedbacks.iter().enumerate() {
-                // Calculate this current stage's percentage
-                let feedback_percent = *feedback as f64 / elapsed;
-
-                // Ignore this feedback if it isn't used
-                if feedback_percent == 0.0 {
-                    continue;
-                }
-
-                // Update the other percent by removing this current percent
-                other_percent -= feedback_percent;
-
-                // Write the percentage for this feedback
-                writeln!(
-                    f,
-                    "    {:6.4}: Feedback index {}",
-                    feedback_percent, feedback_index
-                )?;
-            }
         }
 
-        write!(f, "  Not Measured: {:4.2}", other_percent)?;
+        writeln!(f, "  Feedbacks:")?;
+
+        for (feedback_name, feedback_time) in self.feedbacks() {
+            // Calculate this current stage's percentage
+            let feedback_percent = *feedback_time as f64 / elapsed;
+
+            // Ignore this feedback if it isn't used
+            if feedback_percent == 0.0 {
+                continue;
+            }
+
+            // Update the other percent by removing this current percent
+            other_percent -= feedback_percent;
+
+            // Write the percentage for this feedback
+            writeln!(f, "    {:6.4}: {}", feedback_percent, feedback_name)?;
+        }
+
+        write!(f, "  {:6.4}: Not Measured", other_percent)?;
 
         Ok(())
     }
